==> regex-20.0.0-sources.jar: com/oracle/truffle/regex/tregex/automaton/TransitionSet.java <==
/*
 * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.regex.tregex.automaton;

import com.oracle.truffle.regex.tregex.util.json.JsonConvertible;

import java.util.Collection;

/**
 * Represents a set of NFA transitions to be used in {@link TransitionBuilder}.
 *
 * The set of NFA transitions implicitly also represents a set of NFA <em>target states</em>, which
 * should be used to determine equality between instances of this class.
 */
public interface TransitionSet extends JsonConvertible {

    /**
     * Create a merged set of {@code this} and {@code other} by copying {@code this} and adding the
     * contents of {@code other} to the copy.
     * 
     * @param other the {@link TransitionSet} to be merged with the copy. Implementing classes may
     *            accept objects of their own type only.
     * @return the merged set. <strong>MUST</strong> be of the same type as {@code this}!
     */
    TransitionSet createMerged(TransitionSet other);

    /**
     * Add the contents of {@code other} to {@code this}, analogous to
     * {@link java.util.Set#addAll(Collection)}.
     * 
     * @param other the {@link TransitionSet} to be merged with {@code this}. Implementing classes
     *            may accept objects of their own type only.
     */
    void addAll(TransitionSet other);

    /**
     * Returns the hash code value for this object.
     *
     * The hash should be calculated from the <strong>set of target states</strong> represented by
     * this transition set!
     */
    @Override
    int hashCode();

    /**
     * Checks if this transition set is equal to another.
     *
     * Two transition sets should be treated as equal if and only if their <strong>set of target
     * states</strong> is equal!
     */
    @Override
    boolean equals(Object obj);
}

==> regex-20.1.0-sources.jar: com/oracle/truffle/regex/tregex/automaton/TransitionSet.java <==
/*
 * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.regex.tregex.automaton;

/**
 * Represents a set of NFA transitions to be used in {@link TransitionBuilder}.
 */
public class TransitionSet<SI extends StateIndex<? super S>, S extends AbstractState<S, T>, T extends AbstractTransition<S, T>> {

    private final T[] transitions;
    private final StateSet<SI, S> targetStateSet;

    public TransitionSet(T[] transitions, StateSet<SI, S> targetStateSet) {
        this.transitions = transitions;
        this.targetStateSet = targetStateSet;
    }

    public T[] getTransitions() {
        return transitions;
    }

    public StateSet<SI, S> getTargetStateSet() {
        return targetStateSet;
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public int size() {
        return transitions.length;
    }

    public T getTransition(int i) {
        return transitions[i];
    }
}
